@@ -75,6 +75,8 @@ def estimate_area_dispersion_factor(read, m, n, idx, peaktype, diff_test):
 
 def cal_FDR(peak_list, num_tests):
     '''Calculate the Benjamini-Hochberg FDR'''
+    if len(peak_list)==0:
+        return peak_list
     peak_list = sorted(peak_list, key=attrgetter('pvalue'))
     #calculate BH q-values
     q_list = [item.pvalue*num_tests/(idx+1) 
@@ -128,7 +130,7 @@ def negative_binomial(readData, peakfilename, swap, parameter):
     #sig_index_dict = {}
     sig_peaks_list = []    
 
-    for chr in chr_list:
+    for chr in read_dict:
 
         read_array = read_dict[chr]
         read_array[numpy.where(read_array ==0)] = 1 
@@ -186,7 +188,7 @@ def negative_binomial(readData, peakfilename, swap, parameter):
     # merge adjacent significant peaks. 
     info ("Merging adjacent significant windows...")
     final_peak_list = []
-    for chr in chr_list:
+    for chr in read_dict:
         sig_peak_list_by_chr = \
                 [item for item in sig_peaks_list if item.chr==chr]
         if len(sig_peak_list_by_chr) == 0:
@@ -204,10 +206,10 @@ def negative_binomial(readData, peakfilename, swap, parameter):
 
     # post-processing 
     if remove_artefacts is True: 
-         dump_file = open(peakfilename[:-10]+"_filtered_peaks.txt","w")
+         dump_file = open(peakfilename[:-10]+"_removed_peaks.txt","w")
          dump_file.write("chr\tstart\tend\twidth\tp-value\tq-value\t")
          dump_file.write("chip_input_ratio\ttag_monopoly\t")
-         dump_file.write("strand_overlap_original\tstrand_overlap_shifted")
+         dump_file.write("strand_overlap_original\tstrand_overlap_shifted\n")
     info("begin post-processing...")
     for final_peak in final_peak_list: 
         chr = final_peak[0]
@@ -229,7 +231,8 @@ def negative_binomial(readData, peakfilename, swap, parameter):
                     readData.shift_size, readData.read_length,
                     narrow_peak, remove_artefacts)
             if remove_artefacts and (chip_input_ratio > 0.5 or 
-                    tag_monopoly > 0.5 or (overlap_orig > 0.2 and
+                    # tag_monopoly > 0.5 or  #will not remove tag monopoly cases. 
+                    (overlap_orig > 0.2 and
                 overlap_roll/overlap_orig < 0.5)):
                 dump_file.write(chr+"\t" + str(start) + '\t' + str(end) + '\t' +
                         str(end-start) + '\t' + str(pval) + '\t' + str(qval) + 
@@ -302,31 +305,35 @@ def post_processing_per_peak(strands_dict, chip_list, input_list, chr,
     for chip in chip_list:
         forward = strands_dict[chr][chip]['f']
         reverse = strands_dict[chr][chip]['r']
-        forward_read = forward[numpy.where( (forward > start-shiftSize[chip]) &
-                (forward < end-shiftSize[chip]) )]
-        reverse_read = reverse[numpy.where( (reverse > start+shiftSize[chip]) &
-                (reverse < end+shiftSize[chip]) )]
+        forward_read = forward[numpy.where( (forward >= start) &
+                (forward < end) )]
+        reverse_read = reverse[numpy.where( (reverse >= start+readLength) &
+                (reverse < end+readLength) )]
         for read in forward_read:
-            chip_forward[(read-start)] +=1
+            try: chip_forward[(read-start)] +=1
+            except IndexError:
+                debug("index error ignored. end-start: %d. read-start: %d.", end-start, read-start)
         for read in reverse_read:
             try: chip_reverse[(read-start-readLength)] +=1
             except IndexError:
-                pass
+                debug("index error ignored. end-start: %d. read-start-readLength: %d.", end-start, read-start-readLength)
     # using input reads to remove artefacts
     if remove_artefacts is True: 
         for input in input_list:
             forward = strands_dict[chr][input]['f']
             reverse = strands_dict[chr][input]['r']
-            forward_read = forward[numpy.where( (forward > 
-                    start-shiftSize[chip]) & (forward < end-shiftSize[chip]) )]
-            reverse_read = reverse[numpy.where( (reverse > 
-                    start+shiftSize[chip]) & (reverse < end+shiftSize[chip]) )]
+            forward_read = forward[numpy.where( (forward >= 
+                    start) & (forward < end) )]
+            reverse_read = reverse[numpy.where( (reverse >=
+                    start+readLength) & (reverse < end+readLength) )]
             for read in forward_read:
-                input_forward[(read-start)] +=1
+                try: input_forward[(read-start)] +=1
+                except IndexError: 
+                    debug("index error ignored. end-start: %d. read-start: %d.", end-start, read-start)
             for read in reverse_read:
                 try: input_reverse[(read-start-readLength)] +=1
                 except IndexError:
-                    pass
+                    debug("index error ignored. end-start: %d. read-start-readLength: %d.", end-start, read-start-readLength)
 
         chip_both = chip_forward + chip_reverse
         input_both = input_forward + input_reverse
@@ -411,9 +418,13 @@ def shift_size_per_peak(strands_dict, chip_list, chr, start, end,
         forward = numpy.zeros(end-start)
         reverse = numpy.zeros(end-start)
         for read in forward_read:
-            forward[(read-start):(read-start+readLength)] +=1
+            try: forward[(read-start):(read-start+readLength)] +=1
+            except IndexError:
+                debug("index error ignored. end-start: %d. read-start+readLength: %d.", end-start, read-start+readLength)
         for read in reverse_read:
-            reverse[(read-start-readLength):(read-start)] +=1
+            try: reverse[(read-start-readLength):(read-start)] +=1
+            except IndexError: 
+                debug("index error ignored. end-start: %d. read-start: %d.", end-start, read-start)
         shade_max = 0
         shift_max = 0
         # Iterate from 0 to 250 bp, find the optimum shift that produce 
